package funktio.integroi;

import static java.lang.Math.*;

/**
 * Ohjelmalla integroidaan numeerisesti funktio sin(x) ja 2-asteen polynomi
 * @author Vesa Lappalainen
 * @version 1.0, 07.03.2015
 */
public class Integroi3 {

    /**
     * Rajapinta kaikille funktiolle R->R
     */
    @FunctionalInterface
    public interface FunktioRR {
        /**
         * @param x piste jossa lasketaan
         * @return funktion arvo pisteess
         */
        public double f(double x);
    }


    /**
     * Integroidaan sin(x) vlill x1-x2
     * @param f integroitva funktio-olio
     * @param x1 alkuarvo
     * @param x2 loppuarvo
     * @param tiheys monellako askeleella
     * @return likiarvo integraalille
     */
    public static double integroi(FunktioRR f, double x1, double x2, int tiheys) {
        double summa = 0;
        double dx = (x2 - x1) / tiheys;
        for (double x = x1 + dx / 2; x < x2; x += dx)
            summa += f.f(x) * dx;
        return summa;
    }


    /**
     * @param args ei kytss
     */
    public static void main(String[] args) {
        double ala = integroi(x -> Math.sin(x), 0, PI, 10000);
        System.out.printf("%17.15f%n", ala);
        ala = integroi(x -> x*x + 2*x -3, 0, 3, 100); // x^2 +2x -3 
        System.out.printf("%17.15f%n", ala);
        ala = integroi(x -> x*2 + 2*x, 0, 3, 10000); // x^2 +2x
        System.out.printf("%17.15f%n", ala);
    }

}
