package kombinatiot;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.regex.Pattern;


/**
 * Tehdn kaikki 8 bitin 0/1 merkkijonot sek sellaiset, joissa
 * on 1 tilalla 1.  Sitten verrataan tuottaako mallina oleva regexp saman kuin
 * stdin-virrasta luettu jono.  Jos stdin on tyhj, vertailuun kytetn
 * metodeja tarkistaMalli ja tarkista.
 * Tarkoitettu lhinn TIMist kytettvksi, jolloin isolla kirjoitettuja
 * tekstej voidaan TIMin puolelta vaihtaa toisiksi ennen knnst. 
 * @author vesal
 * @version 18.1.2015
 *
 */
public class Binaarikombinaatiot {
    
    /**
     * Rajapinta funktiolle joka vertaa jonon oikeellisuutta
     */
    public interface Vertaaja {
        /**
         * @param s tutkittava jono
         * @return true jos jono on kelvollinen
         */
        boolean tarkista(String s);
    }

    private static int MAXERRORS = 5;
    private static int JONONPITUUS = 8;
    private static String mallireg = "[01]*00[01]*";   /// MALLIREG
    
    /**
     * Katsotaan saadaan stdin:ist mitn ja jos ei saada, niin kytetn 
     * apumetodeja kyttv vertailua.  Jos saadaan, niin stdinist saatu
     * regexp verrataan mallireg-expin kanssa.
     * @param args ei kytss
     */
    public static void main(String[] args) {
        String verrattava = "";
        // verrattava = "([01]00[01]*)";
        try (Scanner sc = new Scanner(System.in) ) { if ( sc.hasNextLine() ) verrattava = sc.nextLine();  } // INPUT
        
        List<String> jonot = new ArrayList<String>();
        jonot.add("");
        for (int i=0; i<=JONONPITUUS; i++) {
            lisaaJonot(jonot,i);
        }
        
        // for ( String s: jonot) System.out.println(s);
        List<String> errors;
        if ( verrattava.length() > 0 )
            errors = vertaa(jonot, mallireg, verrattava, MAXERRORS);
        else
            errors = vertaa(jonot, (s)->tarkistaMalli(s), (s)->tarkista(s),MAXERRORS);
        
        if ( errors.size() == 0 ) System.out.println("Oikein!");
        else {
            for ( String s: errors) System.out.println(s);
        }
    }
    

    /**
     * Vertaa listan kaikkia jonoja ett tulkitseeko molemmat tarkista-metodit ne samalla tavalla
     * @param jonot lista tutkittavista merkkijonoista
     * @param v1 malli vertaaja-funktio
     * @param v2 tutkittava vertaaja-funktio
     * @param maxErrors monenko virheenjlkeen vertaaminen lopetetaan
     * @return lytyneet virheet, ei enemp kuin maxErrors
     * @example
     * <pre name="test">
     * #import java.util.List;
     * #import java.util.Arrays;
     * #import java.util.regex.Pattern;
     *    List<String> jonot = Arrays.asList("abc","abcd","aaa","baa");
     *    List<String> errors = vertaa(jonot,s->Pattern.matches("a..",s),s->s.charAt(0)=='a',5);
     *    errors.toString() === "[abcd true VRIN]";
     *    errors = vertaa(jonot,s->Pattern.matches("a..",s),s->s.charAt(0)=='a' && s.length() == 3, 5);
     *    errors.size() === 0;
     * </pre>
     */
    public static ArrayList<String> vertaa(List<String> jonot, Vertaaja v1, Vertaaja v2, int maxErrors) {
        ArrayList<String> errors = new ArrayList<String>();
        for (String s: jonot) {
            boolean m1 = v1.tarkista(s);;
            boolean m2 = v2.tarkista(s);
            if (m1 == m2) continue;
            errors.add(String.format("%s %b VRIN",s,m2));
            if (errors.size() >= maxErrors) break;
        }
        return errors;
    }

    
    private static boolean tarkistaMalli(String s) { return Pattern.matches(mallireg, s); } // TARKISTAMALLI

    private static boolean tarkista(String s) { return Pattern.matches("([01]*00[01]*)", s); } // TARKISTAVERTAA


    /**
     * Vertaa listan kaikkia jonoja ett tulkitseeko molemmat regexpit ne samalla tavalla
     * @param jonot lista tutkittavista merkkijonoista
     * @param malli mallina kytettv regexp
     * @param verrattava jono johon verrataan
     * @param maxErrors monenko virheenjlkeen vertaaminen lopetetaan
     * @return lytyneet virheet, ei enemp kuin maxErrors
     * @example
     * <pre name="test">
     * #import java.util.ArrayList;
     *    ArrayList<String> jonot = new ArrayList<String>();
     *    lisaaJonot(jonot,3);
     *    ArrayList<String> errors = vertaa(jonot,"00[01]","00.",5);
     *    errors.toString() === "[00a true VRIN]";
     *    errors = vertaa(jonot,"00[01]","00(0|1)",5);
     *    errors.size() === 0;
     * </pre>
     */
    public static ArrayList<String> vertaa(List<String> jonot, String malli,  String verrattava, int maxErrors) {
        Pattern pm = Pattern.compile(malli);
        Pattern pv = Pattern.compile(verrattava);
        ArrayList<String> errors = new ArrayList<String>();
        for (String s: jonot) {
            boolean m1 = pm.matcher(s).matches();
            boolean m2 = pv.matcher(s).matches();
            if (m1 == m2) continue;
            errors.add(String.format("%s %b VRIN",s,m2));
            if (errors.size() >= maxErrors) break;
        }
        return errors;
    }


    /**
     * Tekee kaikki n mittaiset jonot, joissa on 0,1 kaikki kombinatiot ja niist
     * viel kopion, jossa on 1:n tilalla a. 
     * @param jonot lista johon kombinaatiot listn
     * @param n jonon maksimipituus
     * @example
     * <pre name="test">
     * #import java.util.ArrayList;
     *    ArrayList<String> jonot = new ArrayList<String>();
     *    lisaaJonot(jonot,1);
     *    jonot.toString() === "[0, 1, a]";
     *    jonot.clear();
     *    lisaaJonot(jonot,2);
     *    jonot.toString() === "[00, 01, 0a, 10, a0, 11, aa]";
     *    lisaaJonot(jonot,3);
     *    jonot.toString() =R= ".*000, 001, 00a, 010, 0a0, 011, 0aa, 100, a00, .*";
     * </pre>
     */
    public static void lisaaJonot(List<String> jonot, int n) {
        for (int i=0;;i++) {
            String s = Integer.toBinaryString(i);
            if ( s.length() > n ) break;
            String format ="%"+n+"s";
            s = String.format(format,s);
            s = s.replace(' ', '0');
            jonot.add(s);
            String sa = s.replace('1', 'a');
            if ( !sa.equals(s) ) jonot.add(sa);
        }
    }

}
