package muuttujat.matkaFX;

import java.net.URL;
import java.util.ResourceBundle;

import fi.jyu.mit.ohj2.*;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;


/**
 * Kyttliittymn ksittely matka-ohjelmalle
 * @author vesal
 * @version 17.1.2016
 *
 */
public class MatkaController  implements Initializable {
    @FXML private TextField textMittakaava;
    @FXML private TextField textKartalla;
    @FXML private TextField textMaastossa;

    @FXML
    private void keyReleased() {
        laske();
    }


    @FXML
    private void onEnter(ActionEvent action) {
        ((TextField)(action.getSource())).selectAll();
    }


    @Override
    public void initialize(URL location, ResourceBundle resources) {
        Platform.runLater( () -> textKartalla.requestFocus());
        laske();
    }
	
	
    /** Oletusmittakaava  */
    public static final double MITTAKAAVA = 200000.0;

    /** Kerroin mm muuttamiseksi km */
    public static final double MM_KM = 1000.0 * 1000.0;


    /**
     * Muunnetaan kartalta mm mitattu matka maastoon km
     * @param matka_mm kartalta mitattu matka mm
     * @param mittakaava kytettv mittakaava
     * @return matka maastossa km
     */
    public static double mittakaavamuunnos(double matka_mm, double mittakaava) {
        return mittakaava*matka_mm / MM_KM;
    }


    /**
     * Haetaan tekstikentst luku
     * @param text kentt josta haetaan
     * @param oletus luvun arvo jos kentst ei saa jrkev lukua
     * @return kentst haettu luku
     * @example
     * <pre name="test">
     * #TOLERANCE=0.01
     * #import javafx.embed.swing.JFXPanel;
     * #import javafx.scene.control.*;
     * (new JFXPanel()).toString(); 
     * haeLuku(new TextField("234.233"), 44) ~~~ 234.23;
     * haeLuku(new TextField("foobar1"), 44) ~~~ 44.00;
     * haeLuku(new TextField("1foobar"), 44) ~~~ 1.00;
     * </pre>
     */
    public static double haeLuku(TextField text,double oletus) {
        double luku = Mjonot.erotaDouble(text.getText(), oletus);
        return luku;
    }


    /**
     * Laittaa tekstikenttn tuloksen
     * @param text kentt johon luku laitetaan
     * @param luku joka laitetaan kenttn
     */
    public static void laitaTulos(TextField text, double luku) {
        String tulos = String.format("%5.2f",luku);
        tulos = tulos.replace(',', '.');
        text.setText(tulos);
    }


    /**
     * Lasketaan uudet arvot muuttuneiden perusteella
     */
    private void laske() {
        double mittakaava = haeLuku(textMittakaava,MITTAKAAVA);
        double matka_mm = haeLuku(textKartalla, 0);
        double matka_km = mittakaavamuunnos(matka_mm,mittakaava);
        laitaTulos(textMaastossa,matka_km);
    }
}
