package muuttujat.matkaFX;

import fi.jyu.mit.ohj2.Mjonot;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.RowConstraints;
import javafx.stage.Stage;

/**
 * Matkan muunnokset mittakaavaa kytten.
 * Tehty pelkstn Java-koodilla yhteen tiedostoon.
 * @author vesal
 * @version 23.1.2016
 */
public class MatkaMain2 extends Application {
    @Override
    public void start(Stage primaryStage) {
        try {
            Pane root = luoNaytto();
            Scene scene = new Scene(root); 
            // scene.getStylesheets().add(getClass().getResource("matka.css").toExternalForm());
            primaryStage.setTitle("Matka");
            primaryStage.setScene(scene);
            primaryStage.show();
        } catch(Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * Kynnistetn mittakaavaohjelma 
     * @param args ei kytss
     */
    public static void main(String[] args) {
        launch(args);
    }


    /**
     * Luodaan nyttelementit
     * @return pane jonka pll elementit ovat
     */
    private Pane luoNaytto() {
        BorderPane pane = new BorderPane();
        GridPane grid = new GridPane();
        pane.setCenter(grid);

        ObservableList<ColumnConstraints> cols = grid.getColumnConstraints(); 
        cols.add(new ColumnConstraints(10, 126, 129));
        cols.add(new ColumnConstraints(13, 13, 15));    
        cols.add(new ColumnConstraints(10, 157, 225));    
        cols.add(new ColumnConstraints(10, 45, 82));

        ObservableList<RowConstraints> rows = grid.getRowConstraints();
        rows.add(new RowConstraints(30));
        rows.add(new RowConstraints(30));
        rows.add(new RowConstraints(30));

        Label label;

        grid.add(new Label("Mittakaava"), 0, 0);
        grid.add(new Label("1:"), 1, 0);
        grid.add(new Label("Kartalta mitattu matka"), 0, 1);
        grid.add(new Label("Matka maastossa"), 0, 2);
        grid.add(label = new Label("mm"), 3, 1);  GridPane.setMargin(label, new Insets(0, 0, 0, 6));
        grid.add(label = new Label("km"), 3, 2);  GridPane.setMargin(label, new Insets(0, 0, 0, 6));

        grid.add(textMittakaava = new TextField("200000"), 2, 0);
        textMittakaava.setOnKeyReleased( e -> keyReleased() );
        textMittakaava.setOnAction( e -> onEnter(e) );

        grid.add(textKartalla = new TextField("10"), 2, 1);
        textKartalla.setOnKeyReleased( e -> keyReleased() );
        textKartalla.setOnAction( e -> onEnter(e) );

        grid.add(textMaastossa = new TextField("20"), 2, 2);
        textMaastossa.setEditable(false);
        textMaastossa.setOnAction( e -> onEnter(e) );

        BorderPane.setMargin(grid, new Insets(20, 20, 20 , 20));

        Platform.runLater( () -> textKartalla.requestFocus());
        laske();
        return pane;
    }


    private TextField textMittakaava;
    private TextField textKartalla;
    private TextField textMaastossa;


    private void keyReleased() {
        laske();
    }


    private void onEnter(ActionEvent action) {
        ((TextField)(action.getSource())).selectAll();
    }


    /** Oletusmittakaava  */
    public static final double MITTAKAAVA = 200000.0;

    /** Kerroin mm muuttamiseksi km */
    public static final double MM_KM = 1000.0 * 1000.0;


    /**
     * Muunnetaan kartalta mm mitattu matka maastoon km
     * @param matka_mm kartalta mitattu matka mm
     * @param mittakaava kytettv mittakaava
     * @return matka maastossa km
     */
    public static double mittakaavamuunnos(double matka_mm, double mittakaava) {
        return mittakaava*matka_mm / MM_KM;
    }


    /**
     * Haetaan tekstikentst luku
     * @param text kentt josta haetaan
     * @param oletus luvun arvo jos kentst ei saa jrkev lukua
     * @return kentst haettu luku
     * @example
     * <pre name="test">
     * #TOLERANCE=0.01
     * #import javafx.embed.swing.JFXPanel;
     * #import javafx.scene.control.*;
     * (new JFXPanel()).toString(); 
     * haeLuku(new TextField("234.233"), 44) ~~~ 234.23;
     * haeLuku(new TextField("foobar1"), 44) ~~~ 44.00;
     * haeLuku(new TextField("1foobar"), 44) ~~~ 1.00;
     * </pre>
     */
    public static double haeLuku(TextField text,double oletus) {
        double luku = Mjonot.erotaDouble(text.getText(), oletus);
        return luku;
    }


    /**
     * Laittaa tekstikenttn tuloksen
     * @param text kentt johon luku laitetaan
     * @param luku joka laitetaan kenttn
     */
    public static void laitaTulos(TextField text, double luku) {
        String tulos = String.format("%5.2f",luku);
        tulos = tulos.replace(',', '.');
        text.setText(tulos);
    }


    /**
     * Lasketaan uudet arvot muuttuneiden perusteella
     */
    private void laske() {
        double mittakaava = haeLuku(textMittakaava,MITTAKAAVA);
        double matka_mm = haeLuku(textKartalla, 0);
        double matka_km = mittakaavamuunnos(matka_mm,mittakaava);
        laitaTulos(textMaastossa,matka_km);
    }
}
