package oliot.aika.rajapinta;

/**
 * Minmaalinen luokka joka toteuttaa AikaRajapinnan
 * ja laskee ajan sekunteina 
 * @author vesal
 * @version 15.2.2011
 */
public class AikaSekMin implements AikaRajapinta {

    private int s = 0;
    
    /**
     * @example
     * <pre name="test">
     *   AikaSekMin a = new AikaSekMin();
     *   a.aseta(10,15) === 0; a.toString() === "36900 s";
     *   a.aseta(25,15) === 1; a.toString() === "4500 s";
     * </pre>
     */
    @Override
    public int aseta(int h, int m) {
        s = h*3600 + m*60;
        int vrk = s / (24*3600);
        s = s % (24*3600);
        return vrk;
    }

    
    /**
     * @example
     * <pre name="test">
     *   AikaSekMin a = new AikaSekMin();
     *   a.aseta(10) === 0; a.toString() === "36000 s";
     *   a.aseta(25) === 1; a.toString() === "3600 s";
     * </pre>
     */
    @Override
    public int aseta(int h) {
        return aseta(h,0);
    }

    
    /**
     * @example
     * <pre name="test">
     *   AikaSekMin a = new AikaSekMin();
     *   a.toString() === " 0 s";
     *   a.lisaa(12);
     *   a.toString() === "720 s";
     * </pre>
     */
    @Override
    public int lisaa(int lisattavaMaara) {
        s += lisattavaMaara*60;
        return 0;
    }


    
    /**
     * @example
     * <pre name="test">
     *   AikaSekMin a = new AikaSekMin();
     *   a.aseta(12,50);
     *   a.getH() === 12;
     * </pre>
     */
    @Override
    public int getH() {
        return s / 3600;
    }


    /**
     * @example
     * <pre name="test">
     *   AikaSekMin a = new AikaSekMin();
     *   a.aseta(12,50);
     *   a.getM() === 50;
     * </pre>
     */
    @Override
    public int getM() {
        return (s-getH()*3600)/60;
    }


    /**
     * @example
     * <pre name="test">
     *   AikaSekMin a = new AikaSekMin();
     *   a.aseta(12,50);
     *   a.toString() === "46200 s";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%2d s",s);
    }

    
    /**
     * @param args ei kytss
     */
    public static void main(String[] args) {
        AikaSekMin aika = new AikaSekMin();
        
        System.out.println(aika);
        aika.lisaa(1);
        System.out.println(aika);
        
        AikaRajapinta[] ajat = {new Aika(12,30), new AikaSekMin()};

        for (AikaRajapinta a : ajat) {
            a.lisaa(1);
            System.out.println(a);
        }    
        
    }


}
