package taulukot;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Arrays;

/**
 * Esimerkkej yksiulotteisen taulukon kytst
 * @author Vesa Lappalainen
 * @version 1.0, 04.03.2003
 */
public class Kuut {

    /**
     * Tulostetaan taulukon luvut kukin 2 merkin kokoiseen tilaan
     * vlilynnein eroteltuna.  Loppuun j ylimrinen vlilynti
     * @param os tietovirta johon tulostetaan
     * @param taulukko tulostettava taulukko
     * @example
     * <pre name="test">
     *  #import java.io.*;
     *   ByteArrayOutputStream bs = new ByteArrayOutputStream();
     *   int[] t = {1,2,13,4};
     *   tulosta(bs,t);
     *   bs.toString() =R= "(?s) 1  2 13  4 .*"; // ?s tekee ett \n kelpaa  
     * </pre>
     */
    public static void tulosta(OutputStream os, int taulukko[]) {
        PrintStream out = new PrintStream(os); 
        for (int i = 0; i < taulukko.length; i++)
            out.print(String.format("%2d ",taulukko[i]));
        out.println();
    }


    /**
     * Palautetaan taulukko merkkijonona erottimella eroteltuna. 
     * Lopussa ei ylimrist erotinta.
     * @param taulukko joka palautetaan jonona
     * @param erotin mill jonolla erotellaan
     * @param tila monenko merkin tila kullekin numerolle varataan.
     * @return taulukko merkkijonona erottimella eroteltuna
     * @example
     * <pre name="test">
     * #import java.util.*;
     *   int[] t = {1,2,13,4};
     *   jonoksi(t,",",1) === "1,2,13,4";   
     *   jonoksi(t,",",2) === " 1, 2,13, 4";   
     * </pre>
     */
    public static String jonoksi(int[] taulukko,String erotin,int tila) {
        StringBuilder tulos = new StringBuilder();
        String format = String.format("%%s%%%dd", tila); // esim. %s%2d
        String ero = "";
        for (int  luku : taulukko) {
            tulos.append(String.format(format,ero,luku));
            ero = erotin;
        }
        return tulos.toString();
    }


    /**
     * Esimerkki taulukosta
     * @param args ei kytss
     * @example
     * <pre name="test">
     *   int[] t = {1,2,13,4};
     *   jonoksi(t,",",2)   === " 1, 2,13, 4";   
     *   Arrays.toString(t) === "[1, 2, 13, 4]";
     * </pre>
     */
    public static void main(String[] args) {
        int kPituudet[] = new int[12];
        tulosta(System.out,kPituudet);
                           // 1.  2.  3.  4.  5.  6.  7.  8.  9. 10. 11. 12
        int kPituudet2[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
        tulosta(System.out, kPituudet2);

        int tammikuu = 0;
        int helmikuu = tammikuu + 1;
        int joulukuu = tammikuu + 11;
        kPituudet[3] = 30;
        kPituudet[tammikuu] = 31;
        kPituudet[helmikuu] = 28;
        kPituudet[joulukuu] = 31;
        tulosta(System.out,kPituudet);
        for (int i = 0; i < 12; i++) // tt ei voi tehd foreach-silmukalla
            kPituudet[i] = 0;
        System.out.println(jonoksi(kPituudet,",",2));
        System.out.println(Arrays.toString(kPituudet));
    }
}
