package tiedosto;

import java.util.List;
import fi.jyu.mit.ohj2.Tiedosto;

/**
 * Lukujen lukeminen tiedostosta.
 * Luetaan kaikki kerralla listaan.  Ei hyv jos tiedosto on iso.
 * Esimerkki mys tiedostopohjaisen ohjelman testaamisesta
 * @author Vesa Lappalainen
 * @version 1.0, 03.03.2012
 */
public class TiedKaLista {

    /** 
     * Luetaan tiedosto ja lasketaan siell olevien lukujen
     * summa, keskiarvo ja mr.  Jos tiedoston nime ei
     * anneta, se on luvut.dat
     * @param args tiedoston nimi 
     * @example
     * <pre name="test">
     * #THROWS IOException
     * #import java.io.IOException;
     * #import fi.jyu.mit.ohj2.Suuntaaja;
     * #import fi.jyu.mit.ohj2.VertaaTiedosto;
     * String tiednimi = "tiedka7koe.txt";
     * VertaaTiedosto.kirjoitaTiedosto(tiednimi,"33\n11\nkissa\n5");
     * Suuntaaja.StringOutput so = new Suuntaaja.StringOutput();
     * main(new String[]{tiednimi});
     * so.palauta();
     * String tulos = "Lukuja oli 3 kappaletta.\n"+
     *                "Niiden summa oli 49.00\n"+
     *                "ja keskiarvo oli 16.33\n";  
     * so.ero(tulos) === null;
     * VertaaTiedosto.tuhoaTiedosto(tiednimi);
     * Suuntaaja.StringOutput se = new Suuntaaja.StringOutput(true);
     * main(new String[]{"xxx.xxx"});
     * se.palauta();
     * se.ero("Tiedosto ei aukea! xxx.xxx\n") === null;
     * </pre>
     */
    public static void main(String[] args) {
        double summa = 0;
        int n = 0;
        String tiedNimi = "luvut.dat";
        if (args.length > 0) tiedNimi = args[0];

        List<String> rivit = Tiedosto.lueTiedostoListaan(tiedNimi);
        if (rivit == null) {
            System.err.println("Tiedosto ei aukea! " + tiedNimi);
            return;
        }

        for (String s : rivit)
            try {
                double luku = Double.parseDouble(s);
                summa += luku;
                n++;
            } catch (NumberFormatException ex) {
                // Hyltn
            }

        double ka = 0;
        if (n > 0) ka = summa / n;
        System.out.printf("Lukuja oli %d kappaletta.%n", n);
        System.out.printf("Niiden summa oli %4.2f%n", summa);
        System.out.printf("ja keskiarvo oli %4.2f%n", ka);
    }
}
